# $Id: integrationprocessingsettings.tcl,v 1.1.1.1 2006/08/21 11:19:49 harry Exp $
package provide integrationprocessingsettings 1.0

class IntegrationProcessingSettings {
    inherit itk::Widget Settings2

    # member variables

    private variable block_size_option "0"
    private variable batch_number_check "0"
    private variable mtz_file_option "0"
    private variable mtz_file ""
    

    # member methods

    public method toggleBlockSize
    public method toggleMtzFile
    public method toggleBatchNumber
    
    constructor { args } { }
}

body IntegrationProcessingSettings::constructor { args } {

    
    itk_component add block_size_l {
        gSection $itk_interior.bsl  -text "Block size"
    }

    itk_component add block_size_auto_r {
        SettingRadio $itk_interior.bsar block_size_option -variable [scope block_size_option]  -value 0  -text "Calculate automatically"
    }

    itk_component add block_size_specify_r {
        SettingRadio $itk_interior.bssr block_size_option -variable [scope block_size_option]  -value 1  -text "Specify: " -command [code $this toggleBlockSize]
    }

    itk_component add block_size_e {
        SettingEntry $itk_interior.bse block_size -type int -maximum 20  -minimum 1 -width 5  -justify right  -state disabled
    }

    itk_component add batch_number_l {
        gSection $itk_interior.bnl  -text "Output file"
    }

    itk_component add mtz_file_l {
	label $itk_interior.mfl -text "MTZ file:"
    }

    itk_component add mtz_file_auto_r {
        SettingRadio $itk_interior.mfar mtz_file_option -variable [scope mtz_file_option]  -value 0  -text "Generate MTZ filename automatically"
    }

    itk_component add mtz_file_specify_r {
        SettingRadio $itk_interior.mfsr mtz_file_option -variable [scope mtz_file_option]  -value 1  -text "Specify: "  -command [code $this toggleMtzFile]
    }

    itk_component add mtz_file_fe {
        SettingEntry $itk_interior.mfe \
	    mtz_file \
	    -state disabled
    }

    itk_component add batch_number_c {
        SettingCheckbutton $itk_interior.bnc batch_number_check -text "Apply batch number:" -variable [scope batch_number_check] -command [code $this toggleBatchNumber]
    }

    itk_component add batch_number_e {
        SettingEntry $itk_interior.bne batch_number -type int -maximum 99999  -minimum 0  -width 5  -justify right  -state disabled
    }

    set margin 7
    set indent 20

    grid x $itk_component(block_size_l) - - - x -sticky we
    grid x x $itk_component(block_size_auto_r) - x x -sticky w
    grid x x $itk_component(block_size_specify_r) $itk_component(block_size_e) x x -sticky w
    grid x $itk_component(batch_number_l) - - - x -sticky we
    grid x x $itk_component(mtz_file_auto_r) - x x -sticky w
    grid x x $itk_component(mtz_file_specify_r) $itk_component(mtz_file_fe) x x -sticky w
    grid x x $itk_component(batch_number_c) $itk_component(batch_number_e) x x -sticky w

    grid configure $itk_component(block_size_e) $itk_component(mtz_file_fe) $itk_component(batch_number_e) -sticky w

    grid columnconfigure $itk_interior 0 -minsize $margin
    grid columnconfigure $itk_interior 1 -minsize $indent
    grid columnconfigure $itk_interior 4 -weight 1 
    grid columnconfigure $itk_interior 5 -minsize $margin
    grid rowconfigure $itk_interior 7 -weight 1


    eval itk_initialize $args

}

body IntegrationProcessingSettings::toggleBlockSize { a_value } {
    if {$a_value == 0} {
	$itk_component(block_size_e) configure -state disabled
    } else {
	$itk_component(block_size_e) configure -state normal
    }
}	

body IntegrationProcessingSettings::toggleMtzFile { a_value } {
    if {$a_value == 0} {
	$itk_component(mtz_file_fe) configure -state disabled
    } else {
	$itk_component(mtz_file_fe) configure -state normal
    }
}	

body IntegrationProcessingSettings::toggleBatchNumber { a_value } {
    if {$a_value == 0} {
	$itk_component(batch_number_e) configure -state disabled
    } else {
	$itk_component(batch_number_e) configure -state normal
    }
}	

########################################################################
# Usual config options                                                 #
########################################################################

usual IntegrationProcessingSettings {
   keep -background
   keep -foreground
   keep -selectbackground
   keep -selectforeground
   keep -textbackground
   keep -font
   keep -entryfont
}

