# $Id: integrationrefinementsettings.tcl,v 1.5 2007/07/30 15:48:25 harry Exp $
package provide integrationrefinementsettings 1.0

class IntegrationRefinementSettings {
    inherit itk::Widget Settings2

    # Variables
    private variable cell_ref_fix_beam "0"
    private variable cell_ref_fix_distance "0"
    private variable cell_ref_fix_yscale "0"
    private variable cell_ref_fix_tilt "0"
    private variable cell_ref_fix_twist "0"
    private variable cell_ref_fix_radial_offset "0"
    private variable cell_ref_fix_tangential_offset "0"
    private variable cell_ref_postrefinement_check "1"
    private variable cell_ref_fix_a "0"
    private variable cell_ref_fix_b "0"
    private variable cell_ref_fix_c "0"
    private variable cell_ref_fix_alpha "0"
    private variable cell_ref_fix_beta "0"
    private variable cell_ref_fix_gamma "0"
    private variable cell_ref_fix_mosaicity "0"

    private variable integration_fix_beam "0"
    private variable integration_fix_distance "0"
    private variable integration_fix_yscale "0"
    private variable integration_fix_tilt "0"
    private variable integration_fix_twist "0"
    private variable integration_fix_radial_offset "0"
    private variable integration_fix_tangential_offset "0"
    private variable integration_postrefinement_check "1"
    private variable integration_fix_a "1"
    private variable integration_fix_b "1"
    private variable integration_fix_c "1"
    private variable integration_fix_alpha "1"
    private variable integration_fix_beta "1"
    private variable integration_fix_gamma "1"
    private variable integration_fix_mosaicity "0"

    private method togglePostrefinement
    private method toggleCell
    private method defaultValues

    constructor { args } { }

}

body IntegrationRefinementSettings::constructor { args } {

    itk_component add heading_l {
	label $itk_interior.hl -text "Parameters to fix during processing:" -anchor w
    }
    
    itk_component add cell_refinement_l {
	label $itk_interior.crl -text "Cell\nrefinement:"
    }
    
    itk_component add integration_l {
	label $itk_interior.il -text "Integration:"
    }
    
    itk_component add refinement_l {
	label $itk_interior.rl -text "Positional\nrefinement:  " -justify left -anchor w
    }
    
    itk_component add beam_l {
	label $itk_interior.beaml -text "Beam position:"
    }
    
    itk_component add distance_l {
	label $itk_interior.distl -text "Distance:"
    }

    itk_component add yscale_l {
	label $itk_interior.yscalel -text "Y-scale:"
    }
    
    itk_component add tilt_l {
	label $itk_interior.tiltl -text "Tilt:"
    }
    
    itk_component add twist_l {
	label $itk_interior.twistl -text "Twist:"
    }
    
    itk_component add radial_offset_l {
	label $itk_interior.roffsetl -text "Radial offset:"
    }
    
    itk_component add tangential_offset_l {
	label $itk_interior.toffsetl -text "Tangential offset:"
    }
    
    itk_component add postrefinement_l {
	label $itk_interior.pl -text "Postrefinement:  "
    }
    
    itk_component add postrefine_l {
	label $itk_interior.postrefl -text "Postrefine:"
    }
    
    itk_component add a_l {
	label $itk_interior.lc -text "Cell a:"
    }
    
    itk_component add b_l {
	label $itk_interior.bl -text "Cell b:"
    }
    
    itk_component add c_l {
	label $itk_interior.cl  -text "Cell c:"
    }
    
    itk_component add alpha_l {
	label $itk_interior.alphal  -text "Cell alpha:"
    }
    
    itk_component add beta_l {
	label $itk_interior.betal -text "Cell beta:"
    }
    
    itk_component add gamma_l {
	label $itk_interior.gammal -text "Cell gamma:"
    }
    
    itk_component add mosaicity_l {
	label $itk_interior.mosaicl -text "Mosaicity:"
    }
        
    itk_component add cell_ref_fix_beam_c {
	SettingCheckbutton $itk_interior.cbeamc cell_ref_fix_beam -text "Fix" -variable [scope cell_ref_fix_beam]
    }
    
    itk_component add cell_ref_fix_distance_c {
	SettingCheckbutton $itk_interior.cdistc cell_ref_fix_distance -text "Fix" -variable [scope cell_ref_fix_distance]
    }

    itk_component add cell_ref_fix_yscale_c {
	SettingCheckbutton $itk_interior.cyscalec cell_ref_fix_yscale -text "Fix" -variable [scope cell_ref_fix_yscale]
    }
    
    itk_component add cell_ref_fix_tilt_c {
	SettingCheckbutton $itk_interior.ctiltc cell_ref_fix_tilt  -text "Fix" -variable [scope cell_ref_fix_tilt]
    }
    
    itk_component add cell_ref_fix_twist_c {
	SettingCheckbutton $itk_interior.ctwistc cell_ref_fix_twist -text "Fix" -variable [scope cell_ref_fix_twist]
    }
    
    itk_component add cell_ref_fix_radial_offset_c {
	SettingCheckbutton $itk_interior.croffsetc cell_ref_fix_radial_offset -text "Fix" -variable [scope cell_ref_fix_radial_offset]
    }
    
    itk_component add cell_ref_fix_tangential_offset_c {
	SettingCheckbutton $itk_interior.ctoffsetc cell_ref_fix_tangential_offset -text "Fix" -variable [scope cell_ref_fix_tangential_offset]
    }
    
    itk_component add cell_ref_postrefinement_c {
	SettingCheckbutton $itk_interior.cpostrefc cell_ref_postrefinement_check -variable [scope cell_ref_postrefinement_check] -command [code $this togglePostrefinement cell_ref]
    }
    
    itk_component add cell_ref_fix_a_c {
	SettingCheckbutton $itk_interior.cac cell_ref_fix_a -text "Fix" -variable [scope cell_ref_fix_a]  -command [code $this toggleCell cell_ref a]
    }
    
    itk_component add cell_ref_fix_b_c {
	SettingCheckbutton $itk_interior.cbc cell_ref_fix_b -text "Fix" -variable [scope cell_ref_fix_b]   -command [code $this toggleCell cell_ref b]
    }
    
    itk_component add cell_ref_fix_c_c {
	SettingCheckbutton $itk_interior.ccc cell_ref_fix_c -text "Fix" -variable [scope cell_ref_fix_c]   -command [code $this toggleCell cell_ref c]
    }
    
    itk_component add cell_ref_fix_alpha_c {
	SettingCheckbutton $itk_interior.calphac cell_ref_fix_alpha -text "Fix" -variable [scope cell_ref_fix_alpha]   -command [code $this toggleCell cell_ref alpha]
    }
    
    itk_component add cell_ref_fix_beta_c {
	SettingCheckbutton $itk_interior.cbetac cell_ref_fix_beta -text "Fix" -variable [scope cell_ref_fix_beta]   -command [code $this toggleCell cell_ref beta]
    }
    
    itk_component add cell_ref_fix_gamma_c {
	SettingCheckbutton $itk_interior.cgammac cell_ref_fix_gamma -text "Fix" -variable [scope cell_ref_fix_gamma]   -command [code $this toggleCell cell_ref gamma]
    }
    
    itk_component add cell_ref_cell_linker {
        Linker $itk_interior.ccl  -orient "vertical" -state "closed" -width 10  -pad 1
    }

    itk_component add cell_ref_fix_mosaicity_c {
	SettingCheckbutton $itk_interior.cmosaicc cell_ref_fix_mosaicity -text "Fix" -variable [scope cell_ref_fix_mosaicity] 
    }
        
    itk_component add integration_fix_beam_c {
	SettingCheckbutton $itk_interior.ibeamc integration_fix_beam -text "Fix" -variable [scope integration_fix_beam]
    }
    
    itk_component add integration_fix_distance_c {
	SettingCheckbutton $itk_interior.idistc integration_fix_distance -text "Fix" -variable [scope integration_fix_distance]
    }

    itk_component add integration_fix_yscale_c {
	SettingCheckbutton $itk_interior.iyscalec integration_fix_yscale -text "Fix" -variable [scope integration_fix_yscale]
    }
    
    itk_component add integration_fix_tilt_c {
	SettingCheckbutton $itk_interior.itiltc integration_fix_tilt -text "Fix" -variable [scope integration_fix_tilt]
    }
    
    itk_component add integration_fix_twist_c {
	SettingCheckbutton $itk_interior.itwistc integration_fix_twist -text "Fix" -variable [scope integration_fix_twist]
    }
    
    itk_component add integration_fix_radial_offset_c {
	SettingCheckbutton $itk_interior.iroffsetc integration_fix_radial_offset -text "Fix" -variable [scope integration_fix_radial_offset]
    }
    
    itk_component add integration_fix_tangential_offset_c {
	SettingCheckbutton $itk_interior.itoffsetc integration_fix_tangential_offset -text "Fix" -variable [scope integration_fix_tangential_offset]
    }
    
    itk_component add integration_postrefinement_c {
	SettingCheckbutton $itk_interior.ipostrefc integration_postrefinement_check -variable [scope integration_postrefinement_check] -command [code $this togglePostrefinement integration]
    }
    
    itk_component add integration_fix_a_c {
	SettingCheckbutton $itk_interior.iac integration_fix_a -text "Fix" -variable [scope integration_fix_a]  -command [code $this toggleCell integration a]
    }
    
    itk_component add integration_fix_b_c {
	SettingCheckbutton $itk_interior.ibc integration_fix_b -text "Fix" -variable [scope integration_fix_b]    -command [code $this toggleCell integration b]
    }
    
    itk_component add integration_fix_c_c {
	SettingCheckbutton $itk_interior.icc integration_fix_c -text "Fix" -variable [scope integration_fix_c]   -command [code $this toggleCell integration c]
    }
    
    itk_component add integration_fix_alpha_c {
	SettingCheckbutton $itk_interior.ialphac integration_fix_alpha -text "Fix" -variable [scope integration_fix_alpha]   -command [code $this toggleCell integration alpha]
    }
    
    itk_component add integration_fix_beta_c {
	SettingCheckbutton $itk_interior.ibetac integration_fix_beta -text "Fix" -variable [scope integration_fix_beta]   -command [code $this toggleCell integration beta]
    }
    
    itk_component add integration_fix_gamma_c {
	SettingCheckbutton $itk_interior.igammac integration_fix_gamma -text "Fix" -variable [scope integration_fix_gamma]   -command [code $this toggleCell integration gamma]
    }
    
    itk_component add integration_cell_linker {
        Linker $itk_interior.icl  -orient "vertical" -state "closed" -width 10  -pad 1
    }

    itk_component add integration_fix_mosaicity_c {
	SettingCheckbutton $itk_interior.imosaicc integration_fix_mosaicity -text "Fix" -variable [scope integration_fix_mosaicity]  
    }
       
    itk_component add default_button {
	button $itk_interior.default \
	    -text "Defaults" \
	    -command [code $this defaultValues]
    }
 
    set margin 7
    set indent 20

    grid $itk_component(heading_l) - - - -sticky nw  -pady $margin
    grid x x $itk_component(cell_refinement_l) - $itk_component(integration_l) - -sticky nw -padx $margin -pady $margin

    grid $itk_component(refinement_l) $itk_component(beam_l) $itk_component(cell_ref_fix_beam_c) x $itk_component(integration_fix_beam_c) -sticky nw
    grid ^ $itk_component(distance_l) $itk_component(cell_ref_fix_distance_c) x $itk_component(integration_fix_distance_c) -sticky nw
    grid ^ $itk_component(yscale_l) $itk_component(cell_ref_fix_yscale_c) x $itk_component(integration_fix_yscale_c) -sticky nw
    grid ^ $itk_component(tilt_l) $itk_component(cell_ref_fix_tilt_c) x $itk_component(integration_fix_tilt_c) -sticky nw
    grid ^ $itk_component(twist_l) $itk_component(cell_ref_fix_twist_c) x $itk_component(integration_fix_twist_c) -sticky nw
    grid ^ $itk_component(radial_offset_l) $itk_component(cell_ref_fix_radial_offset_c) x $itk_component(integration_fix_radial_offset_c) -sticky nw
    grid ^ $itk_component(tangential_offset_l) $itk_component(cell_ref_fix_tangential_offset_c) x $itk_component(integration_fix_tangential_offset_c) -sticky nw

    grid $itk_component(postrefinement_l) $itk_component(postrefine_l) $itk_component(cell_ref_postrefinement_c) x $itk_component(integration_postrefinement_c) -sticky nw -pady $margin

    grid x $itk_component(a_l) $itk_component(cell_ref_fix_a_c) $itk_component(cell_ref_cell_linker) $itk_component(integration_fix_a_c) $itk_component(integration_cell_linker) x -sticky nw

    grid x $itk_component(b_l) $itk_component(cell_ref_fix_b_c) ^ $itk_component(integration_fix_b_c) ^ -sticky nw
    grid x $itk_component(c_l) $itk_component(cell_ref_fix_c_c) ^ $itk_component(integration_fix_c_c) ^ -sticky nw
    grid x $itk_component(alpha_l) $itk_component(cell_ref_fix_alpha_c) ^ $itk_component(integration_fix_alpha_c) ^ -sticky nw
    grid x $itk_component(beta_l) $itk_component(cell_ref_fix_beta_c) ^ $itk_component(integration_fix_beta_c) ^ -sticky nw
    grid x $itk_component(gamma_l) $itk_component(cell_ref_fix_gamma_c) ^ $itk_component(integration_fix_gamma_c) ^ -sticky nw
    grid x $itk_component(mosaicity_l) $itk_component(cell_ref_fix_mosaicity_c) x $itk_component(integration_fix_mosaicity_c) x -sticky nw

    grid $itk_component(default_button) - - - - -  -sticky e -pady $margin -padx $margin

    grid configure $itk_component(cell_ref_cell_linker) -sticky nsw
    grid configure $itk_component(integration_cell_linker) -sticky nsw

    grid configure $itk_component(cell_ref_fix_beam_c) $itk_component(integration_fix_beam_c) $itk_component(cell_ref_fix_distance_c) $itk_component(integration_fix_distance_c) $itk_component(cell_ref_fix_yscale_c) $itk_component(integration_fix_yscale_c) $itk_component(cell_ref_fix_tilt_c) $itk_component(integration_fix_tilt_c) $itk_component(cell_ref_fix_twist_c) $itk_component(integration_fix_twist_c) $itk_component(cell_ref_fix_radial_offset_c) $itk_component(integration_fix_radial_offset_c) $itk_component(cell_ref_fix_tangential_offset_c) $itk_component(integration_fix_tangential_offset_c) $itk_component(cell_ref_postrefinement_c) $itk_component(integration_postrefinement_c) $itk_component(cell_ref_fix_a_c) $itk_component(integration_fix_a_c) $itk_component(cell_ref_fix_b_c) $itk_component(integration_fix_b_c) $itk_component(cell_ref_fix_c_c) $itk_component(integration_fix_c_c) $itk_component(cell_ref_fix_alpha_c) $itk_component(integration_fix_alpha_c) $itk_component(cell_ref_fix_beta_c) $itk_component(integration_fix_beta_c) $itk_component(cell_ref_fix_gamma_c) $itk_component(integration_fix_gamma_c) $itk_component(cell_ref_fix_mosaicity_c) $itk_component(integration_fix_mosaicity_c) -sticky e

#     grid columnconfigure $itk_interior 0 -minsize $margin -weight 0
#     grid columnconfigure $itk_interior { 1 2 } -minsize $indent -weight 0
#     grid columnconfigure $itk_interior 4 -weight 1
#     grid rowconfigure $itk_interior 8 -minsize 14
#     grid rowconfigure $itk_interior 13 -weight 1
}

body IntegrationRefinementSettings::togglePostrefinement { a_process a_value } {
    if {$a_value == 0} {
	$itk_component(${a_process}_fix_a_c) configure -state disabled
	$itk_component(${a_process}_fix_b_c) configure -state disabled
	$itk_component(${a_process}_fix_c_c) configure -state disabled
	$itk_component(${a_process}_fix_alpha_c) configure -state disabled
	$itk_component(${a_process}_fix_beta_c) configure -state disabled
	$itk_component(${a_process}_fix_gamma_c) configure -state disabled
	$itk_component(${a_process}_fix_mosaicity_c) configure -state disabled
    } else {
	$itk_component(${a_process}_fix_a_c) configure -state normal
	$itk_component(${a_process}_fix_b_c) configure -state normal
	$itk_component(${a_process}_fix_c_c) configure -state normal
	$itk_component(${a_process}_fix_alpha_c) configure -state normal
	$itk_component(${a_process}_fix_beta_c) configure -state normal
	$itk_component(${a_process}_fix_gamma_c) configure -state normal
	$itk_component(${a_process}_fix_mosaicity_c) configure -state normal
    }
}

body IntegrationRefinementSettings::toggleCell { a_process a_param a_value } {
    if {[$itk_component(${a_process}_cell_linker) query]} {
	foreach i_param { a b c alpha beta gamma } {
	    if {$i_param != $a_param} {
		set ${a_process}_fix_${i_param} $a_value
	    }
	}
    } else {
    }
}

body IntegrationRefinementSettings::defaultValues { } {
    set cell_ref_fix_beam "0"
    set cell_ref_fix_distance "0"
    set cell_ref_fix_yscale "0"
    set cell_ref_fix_tilt "0"
    set cell_ref_fix_twist "0"
    if { $spiral } {
	set cell_ref_fix_radial_offset "1"
	set cell_ref_fix_tangential_offset "1"
    } {
	set cell_ref_fix_radial_offset "0"
	set cell_ref_fix_tangential_offset "0"
    }
    set cell_ref_postrefinement_check "1"
    set cell_ref_fix_a "0"
    set cell_ref_fix_b "0"
    set cell_ref_fix_c "0"
    set cell_ref_fix_alpha "0"
    set cell_ref_fix_beta "0"
    set cell_ref_fix_gamma "0"
    set cell_ref_fix_mosaicity "0"

    set integration_fix_beam "0"
    set integration_fix_distance "0"
    set integration_fix_yscale "0"
    set integration_fix_tilt "0"
    set integration_fix_twist "0"
    if { $spiral } {
	set integration_fix_radial_offset "1"
	set integration_fix_tangential_offset "1"
    } {
	set integration_fix_radial_offset "0"
	set integration_fix_tangential_offset "0"
    }
    set integration_postrefinement_check "1"
    set integration_fix_a "1"
    set integration_fix_b "1"
    set integration_fix_c "1"
    set integration_fix_alpha "1"
    set integration_fix_beta "1"
    set integration_fix_gamma "1"
    set integration_fix_mosaicity "0"
}

########################################################################
# Usual config options                                                 #
########################################################################

usual IntegrationRefinementSettings { 
#    keep -background
#    keep -foreground
#    keep -selectbackground
#    keep -selectforeground
#    keep -textbackground
#    keep -font
#    keep -entryfont
}

