# $Id: integrationreflectionsettings.tcl,v 1.1.1.1 2006/08/21 11:19:50 harry Exp $
package provide integrationreflectionsettings 1.0

class IntegrationReflectionSettings {
    inherit itk::Widget Settings2

    # member variables

    private variable low_resolution_limit_option "0"
    private variable high_resolution_limit_check "0"
    private variable resolution_cutoff_check "0"
    private variable minimum_spot_separation ""
    private variable null_pixel_threshold ""
    private variable int_min_spot_sep_option "0"

    # member methods

    public method toggleLowResLimit
    public method toggleHighResLimit
    public method toggleResCutoff
    public method toggleExclusionRings
    public method toggleMinSpotSeparation
    
    constructor { args } { }
}

body IntegrationReflectionSettings::constructor { args } {

    itk_component add spots_label {
        gSection $itk_interior.sl  -text "Resolution limits"
    }
    
    itk_component add high_res_c {
        SettingCheckbutton $itk_interior.hrc high_resolution_limit_check \
	    -text "Apply high resolution limit: " \
	    -command [code $this toggleHighResLimit]
    }

    itk_component add max_res_e {
        SettingEntry $itk_interior.maxre high_resolution_limit \
	    -type real \
	    -precision 2 \
	    -width 8 \
	    -justify right \
	    -disabledforeground "\#dcdcdc" \
	    -state "disabled"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add low_res_c {
        SettingCheckbutton $itk_interior.lrc low_resolution_limit_check \
	    -text "Apply low resolution limit: " \
	    -command [code $this toggleLowResLimit]
    }

    itk_component add min_res_e {
        SettingEntry $itk_interior.minre low_resolution_limit \
	    -type real \
	    -precision 2 \
	    -width 8 \
	    -justify right \
	    -disabledforeground "\#dcdcdc" \
	    -state "disabled"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add res_cutoff_c {
        SettingCheckbutton $itk_interior.rcc resolution_cutoff_check \
	    -variable [scope resolution_cutoff_check] \
	    -text "Apply I/\u3c3(I) cutoff: " \
	    -command [code $this toggleResCutoff]
    }

    itk_component add res_cutoff_e {
        SettingEntry $itk_interior.rce resolution_cutoff \
	    -type real \
	    -precision 1 \
	    -minimum 0 \
	    -width 8 \
	    -justify right \
	    -state disabled \
	    -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }
    
    itk_component add num_excl_l {
	label $itk_interior.nel -text "Number of exclusion rings: " \
	    -anchor w
    }
    itk_component add exclusions_combo {
	SettingCombo $itk_interior.ec num_exclusion_rings \
	    -width 1 \
	    -items {0 1 2 3 4 5} \
	    -editable 0 \
	    -highlightcolor black \
	    -command [code $this toggleExclusionRings]
    }
    $itk_component(exclusions_combo) setValue 0
    
    itk_component add excl_1_l {
	label $itk_interior.e1l -text "1: "
    }

    itk_component add excl_2_l {
	label $itk_interior.e2l -text "2: "
    }

    itk_component add excl_3_l {
	label $itk_interior.e3l -text "3: "
    }

    itk_component add excl_4_l {
	label $itk_interior.e4l -text "4: "
    }

    itk_component add excl_5_l {
	label $itk_interior.e5l -text "5: "
    }

    itk_component add excl_1_lower_e {
        SettingEntry $itk_interior.e1le excl_1_lower \
	    -type real \
	    -precision 2 \
	    -minimum 0.1 \
	    -maximum 10000 \
	    -width 8 \
	    -justify right \
	    -state "disabled" \
	    -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_1_upper_e {
        SettingEntry $itk_interior.e1ue excl_1_upper \
	    -type real \
	    -precision 2 \
	    -minimum 0.1 \
	    -maximum 10000 \
	    -width 8 \
	    -justify right \
	    -state "disabled" \
	    -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_2_lower_e {
        SettingEntry $itk_interior.e2le excl_2_lower -type real  -precision 2  -minimum 0.1 -maximum 10000 -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_2_upper_e {
        SettingEntry $itk_interior.e2ue excl_2_upper -type real  -precision 2  -minimum 0.1 -maximum 10000  -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_3_lower_e {
        SettingEntry $itk_interior.e3le  excl_3_lower -type real  -precision 2  -minimum 0.1 -maximum 10000  -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_3_upper_e {
        SettingEntry $itk_interior.e3ue excl_3_upper -type real  -precision 2  -minimum 0.1 -maximum 10000  -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_4_lower_e {
        SettingEntry $itk_interior.e4le excl_4_lower -type real  -precision 2  -minimum 0.1 -maximum 10000  -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_4_upper_e {
        SettingEntry $itk_interior.e4ue excl_4_upper -type real  -precision 2  -minimum 0.1 -maximum 10000  -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_5_lower_e {
        SettingEntry $itk_interior.e5le excl_5_lower -type real  -precision 2  -minimum 0.1 -maximum 10000  -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add excl_5_upper_e {
        SettingEntry $itk_interior.e5ue excl_5_upper  -type real  -precision 2  -minimum 0.1 -maximum 10000 -width 8  -justify right  -state "disabled"  -disabledforeground "\#dcdcdc"
    } {
        usual
        ignore -disabledforeground
    }

    itk_component add peak_sep_l {
        gSection $itk_interior.psl  -text "Minimum spot separation"
    }

    itk_component add peak_sep_auto_r {
        SettingRadio $itk_interior.psar int_min_spot_sep_option -variable [scope int_min_spot_sep_option]  -value 0  -text "Calculate dynamically"
    }

    itk_component add peak_sep_specify_r {
        SettingRadio $itk_interior.pssr int_min_spot_sep_option -variable [scope int_min_spot_sep_option]  -value 1  -text "Specify: " -command [code $this toggleMinSpotSeparation]
    }

    itk_component add peak_sep_x_l {
        label $itk_interior.psxl  -text "X (mm):"  -anchor w
    }

    itk_component add peak_sep_y_l {
        label $itk_interior.psyl  -text "Y (mm):"  -anchor w
    }

    itk_component add peak_sep_x_e {
        SettingEntry $itk_interior.psxe int_min_spot_sep_x  -type real  -precision 2  -maximum 10.00  -minimum 0.00 -width 5  -justify right  -linkcommand [code $this linkPeakSep x y]  -state disabled
    }

    itk_component add peak_sep_y_e {
        SettingEntry $itk_interior.psye int_min_spot_sep_y  -type real  -precision 2  -maximum 10.00  -minimum 0.00  -width 4  -justify right  -linkcommand [code $this linkPeakSep y x]  -state disabled
    }

    itk_component add peak_sep_prop_linker {
        Linker $itk_interior.pspl  -orient "vertical"  -width 10  -pad 1  }

     



    set margin 7
    set indent 20

    grid x $itk_component(spots_label) - - - - - x -sticky we
    grid x x $itk_component(high_res_c) - - $itk_component(max_res_e) x x -sticky w
    grid x x $itk_component(low_res_c) - - $itk_component(min_res_e) x x -sticky w
    grid x x $itk_component(res_cutoff_c) - - $itk_component(res_cutoff_e) x x -sticky w
    
    grid x x $itk_component(num_excl_l) - - $itk_component(exclusions_combo) x x -sticky we
    grid x x x $itk_component(excl_1_l) $itk_component(excl_1_lower_e) $itk_component(excl_1_upper_e) x x 
    grid x x x $itk_component(excl_2_l) $itk_component(excl_2_lower_e) $itk_component(excl_2_upper_e) x x
    grid x x x $itk_component(excl_3_l) $itk_component(excl_3_lower_e) $itk_component(excl_3_upper_e) x x 
    grid x x x $itk_component(excl_4_l) $itk_component(excl_4_lower_e) $itk_component(excl_4_upper_e) x x 
    grid x x x $itk_component(excl_5_l) $itk_component(excl_5_lower_e) $itk_component(excl_5_upper_e) x x

    grid x $itk_component(peak_sep_l) - - - - - - - -sticky we
    grid x x $itk_component(peak_sep_auto_r) - - - - - -sticky we
    grid x x $itk_component(peak_sep_specify_r) $itk_component(peak_sep_x_l) - $itk_component(peak_sep_x_e) $itk_component(peak_sep_prop_linker) x -sticky we
    grid x x x $itk_component(peak_sep_y_l) - $itk_component(peak_sep_y_e) ^ x -sticky we
    grid configure $itk_component(peak_sep_prop_linker) -sticky wns

    grid columnconfigure $itk_interior 0 -minsize $margin
    grid columnconfigure $itk_interior 1 -minsize $indent
    grid columnconfigure $itk_interior 6 -weight 1
    grid columnconfigure $itk_interior 7 -minsize $margin
}

body IntegrationReflectionSettings::toggleLowResLimit { a_value } {
    if {$a_value == 0} {
	$itk_component(min_res_e) configure -state disabled
    } else {
	$itk_component(min_res_e) configure -state normal
    }
}	

body IntegrationReflectionSettings::toggleHighResLimit { a_value } {
    if {$a_value == 0} {
	$itk_component(max_res_e) configure -state disabled
    } else {
	$itk_component(max_res_e) configure -state normal
    }
}	

body IntegrationReflectionSettings::toggleResCutoff { a_value } {
    if {$a_value == 0} {
	$itk_component(res_cutoff_e) configure -state disabled
    } else {
	$itk_component(res_cutoff_e) configure -state normal
    }
}	

body IntegrationReflectionSettings::toggleExclusionRings { a_value } {
    $itk_component(excl_1_lower_e) configure -state disabled
    $itk_component(excl_1_upper_e) configure -state disabled
    $itk_component(excl_2_lower_e) configure -state disabled
    $itk_component(excl_2_upper_e) configure -state disabled
    $itk_component(excl_3_lower_e) configure -state disabled
    $itk_component(excl_3_upper_e) configure -state disabled
    $itk_component(excl_4_lower_e) configure -state disabled
    $itk_component(excl_4_upper_e) configure -state disabled
    $itk_component(excl_5_lower_e) configure -state disabled
    $itk_component(excl_5_upper_e) configure -state disabled
    if {$a_value > 0} {
	$itk_component(excl_1_lower_e) configure -state normal
	$itk_component(excl_1_upper_e) configure -state normal
    }
    if {$a_value > 1} {
	$itk_component(excl_2_lower_e) configure -state normal
	$itk_component(excl_2_upper_e) configure -state normal
    }
    if {$a_value > 2} {
	$itk_component(excl_3_lower_e) configure -state normal
	$itk_component(excl_3_upper_e) configure -state normal
    }
    if {$a_value > 3} {
	$itk_component(excl_4_lower_e) configure -state normal
	$itk_component(excl_4_upper_e) configure -state normal
    }
    if {$a_value > 4} {
	$itk_component(excl_5_lower_e) configure -state normal
	$itk_component(excl_5_upper_e) configure -state normal
    }
}	

body IntegrationReflectionSettings::toggleMinSpotSeparation { value } {
    if {$value == 0} {
	#set int_min_spot_sep_x "0.00"
	#set int_min_spot_sep_y "0.00"
	$itk_component(peak_sep_x_e) configure -state disabled
	$itk_component(peak_sep_y_e) configure -state disabled
	$itk_component(peak_sep_x_l) configure -state disabled
	$itk_component(peak_sep_y_l) configure -state disabled
    } else {
	$itk_component(peak_sep_x_e) configure -state normal
	$itk_component(peak_sep_y_e) configure -state normal
	$itk_component(peak_sep_x_l) configure -state normal
	$itk_component(peak_sep_y_l) configure -state normal
    }
}


########################################################################
# Usual config options                                                 #
########################################################################

usual IntegrationReflectionSettings {
   keep -background
   keep -foreground
   keep -selectbackground
   keep -selectforeground
   keep -textbackground
   keep -font
   keep -entryfont
}

