# $Id: spotfinding.tcl,v 1.1.1.1 2006/08/21 11:19:54 harry Exp $
package provide spotfinding 1.0

option add *Spotfinding.relief raised widgetDefault
option add *Spotfinding.borderwidth 2 widgetDefault

class Spotfinding {
    inherit itk::Widget 

    # Layout variables
    private variable margin 7
    private variable indent 20

    # Array of spotfinding results received from mosflm
    private variable req_search_results ; # NB array - don't initialize

    # List of images to be indexed
    private variable images_list {}
    private variable image_objects_by_number ; # NB array - don't initialize
    private variable image_objects_by_item ; # NB array - don't initialize
    private variable image_items_by_number ; # NB array - don't initialize
    private variable image_items_by_object ; # NB array - don't initialize

    private variable choosing_images "0"
    private variable chosen_images_text ""
    private variable chosen_search_images ""
    # List of images being searched
    private variable images_being_searched {}

    # Do job (spot finding, indexing and refinement)
    public method launch
    private method findSpots
    private method findSpotsIfNecessary

    private method pickFirstImage
    private method pickNinetyDegreeImages
    private method chooseImages
    private method chooseNextImage

    # Process results
    public method processSpotfindingResults

    # Spot finding methods
    private method buildImageList
    private method buildImageTree
    private method imageTreeClick
    private method imageTreeDoubleClick
    private method checkSpotlistInclusion
    private method uncheckSpotlistInclusion
    private method toggleSpotlistInclusion
    private method toggleImageSelection
    public method updateSpotlists
    public method updateSpotFindingResult
    public method sortSpotFindingResults
    public method editSpots
    public method createImageCheck
    private method updateSpotlistSelection
    private method updateSpotlistInclusions
    private method updateSummaryLabels

    constructor {args} { }

}

body Spotfinding::constructor { args } {

    itk_option add hull.width hull.height
        
    # Spot finding panel
    ##########################################################
    
    itk_component add spotfindinglabel {
	label $itk_interior.sfl \
	    -text "Spot finding results:"
    }

    itk_component add image_tree {
	treectrl $itk_interior.itree \
	    -showroot 0 \
	    -showline 0 \
	    -showbutton 0 \
	    -selectmode single \
	    -width 430 \
	    -itemheight 18
    } {
	rename -background -textbackground textBackground Background
	rename -font -entryfont entryFont Font
    }

    $itk_component(image_tree) column create -text Image -justify left -minwidth 100 -expand 1 ;#-itembackground {"\#ffffff" "\#e8e8e8"}
    $itk_component(image_tree) column create -text "Auto" -justify center -minwidth 60 -expand 1 
    $itk_component(image_tree) column create -text "Manual" -justify center -minwidth 60 -expand 1
    $itk_component(image_tree) column create -text "Deleted" -justify center -minwidth 60 -expand 1
    $itk_component(image_tree) column create -text "Total" -justify center -minwidth 60 -expand 1
    $itk_component(image_tree) column create -text "Search"  -justify center -minwidth 60 -tag search
    $itk_component(image_tree) column create -text "Use"  -justify center -minwidth 30 -tag use

    $itk_component(image_tree) state define CHECKED
    $itk_component(image_tree) state define AVAILABLE

    $itk_component(image_tree) element create e_icon image -image ::img::plain_image
    $itk_component(image_tree) element create e_text text -fill {white selected}
    $itk_component(image_tree) element create e_highlight rect -showfocus yes -fill { \#3399ff {selected focus} gray {selected !focus} }
    $itk_component(image_tree) element create e_auto_search image -image { ::img::spot_search_auto {} }
    $itk_component(image_tree) element create e_manual_search image -image { ::img::spot_search_manual {} }
    $itk_component(image_tree) element create e_check image -image { ::img::embed_check_on {CHECKED AVAILABLE} ::img::embed_check_off {AVAILABLE !CHECKED} ::img::embed_check_off_disabled {!AVAILABLE} }
	
    $itk_component(image_tree) style create s1
    $itk_component(image_tree) style elements s1 { e_highlight e_icon e_text }
    $itk_component(image_tree) style layout s1 e_icon -expand ns -padx {0 6}
    $itk_component(image_tree) style layout s1 e_text -expand ns
    $itk_component(image_tree) style layout s1 e_highlight -union [list e_icon e_text] -iexpand nse -ipadx 2
    
    $itk_component(image_tree) style create s2
    $itk_component(image_tree) style elements s2 {e_highlight e_text}
    $itk_component(image_tree) style layout s2 e_text -expand ns
    $itk_component(image_tree) style layout s2 e_highlight -union [list e_text] -iexpand nsew -ipadx 2

    $itk_component(image_tree) style create s3
    $itk_component(image_tree) style elements s3 {e_highlight e_auto_search e_manual_search}
    $itk_component(image_tree) style layout s3 e_highlight -union [list e_auto_search e_manual_search] -iexpand nsew -ipadx 2
    $itk_component(image_tree) style layout s3 e_auto_search -expand ns -padx {2 2}
    $itk_component(image_tree) style layout s3 e_manual_search -expand ns -padx {2 2}

    $itk_component(image_tree) style create s4
    $itk_component(image_tree) style elements s4 {e_highlight e_check}
    $itk_component(image_tree) style layout s4 e_highlight -union [list e_check] -iexpand nsew -ipadx 2
    $itk_component(image_tree) style layout s4 e_check -expand ns -padx {2 2}

    bind $itk_component(image_tree) <ButtonPress-1> [code $this imageTreeClick %W %x %y]
    bind $itk_component(image_tree) <Double-ButtonPress-1> [code $this imageTreeDoubleClick %W %x %y]
    bind $itk_component(image_tree) <ButtonRelease-1> { break }

    $itk_component(image_tree) notify bind $itk_component(image_tree) <Selection> [code $this toggleImageSelection %S %D]

    itk_component add image_scroll {
	scrollbar $itk_interior.iscroll \
	    -command [code $this component image_tree yview] \
	    -orient vertical
    }
    
    $itk_component(image_tree) configure \
	-yscrollcommand [list autoscroll $itk_component(image_scroll)]
     
    itk_component add spotselectionlabel {
	label $itk_interior.ssl \
	    -text "Spots to index:"
    }

    itk_component add single_image_button {
	button $itk_interior.sib \
	    -text "1st" \
	    -width 3 \
	    -padx 0 \
	    -pady 0 \
	    -command [code $this pickFirstImage]
    }
    
    itk_component add ninety_degrees_button {
	button $itk_interior.ndb \
	    -text "90\u00b0" \
	    -width 3 \
	    -padx 0 \
	    -pady 0 \
	    -command [code $this pickNinetyDegreeImages] \
	    -state disabled
    }
    
    itk_component add image_numbers_e {
	gEntry $itk_interior.ine \
	    -textvariable [scope chosen_images_text] \
	    -command {}
    }

    bind [$itk_component(image_numbers_e) component entry] <Return> [code $this chooseImages]
    
    itk_component add spotselectionframe {
	frame $itk_interior.ssf \
	    -bd 2 \
	    -relief sunken
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add spotcanvas {
	canvas $itk_interior.ssf.spotcanvas \
	    -width 252 \
	    -height 252 \
	    -borderwidth 1 \
	    -relief solid \
	    -highlightthickness 0
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_images_l {
	label $itk_interior.ssf.sfil \
	    -text "Images used:"
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_auto_l {
	label $itk_interior.ssf.sfal \
	    -text "Spots found:"
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_manual_l {
	label $itk_interior.ssf.sfml \
	    -text "Spots added:"
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_del_l {
	label $itk_interior.ssf.sfdl \
	    -text "Spots deleted:"
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_total_l {
	label $itk_interior.ssf.sftl \
	    -text "Total:"
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_images_r {
	label $itk_interior.ssf.sfir \
	    -width 6 \
	    -anchor e
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_auto_r {
	label $itk_interior.ssf.sfar \
	    -width 6 \
	    -anchor e
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_manual_r {
	label $itk_interior.ssf.sfmr \
	    -width 6 \
	    -anchor e
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_del_r {
	label $itk_interior.ssf.sfdr \
	    -width 6 \
	    -anchor e
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    itk_component add sf_total_r {
	label $itk_interior.ssf.sftr \
	    -width 6 \
	    -anchor e
    } {
	usual
	rename -background -textbackground textBackground Background
    }

    # Layout
    ###############################################################
	
    grid x $itk_component(spotfindinglabel) - x $itk_component(spotselectionlabel) - - x -sticky w
    grid x $itk_component(image_tree) $itk_component(image_scroll) x $itk_component(single_image_button) $itk_component(ninety_degrees_button) $itk_component(image_numbers_e) x -sticky nsew
    grid x ^ ^ x $itk_component(spotselectionframe) - - x -sticky nswe
    grid rowconfigure $itk_component(hull) { 2 } -weight 1
    grid columnconfigure $itk_component(hull) { 0 3 7 } -minsize $margin

    grid $itk_component(spotcanvas) -row 1 -column 1 -columnspan 5 -sticky nw
    grid $itk_component(sf_images_l) -row 3 -column 2 -sticky w
    grid $itk_component(sf_images_r) -row 3 -column 4 -sticky e
    grid $itk_component(sf_auto_l) -row 5 -column 2 -sticky w
    grid $itk_component(sf_auto_r) -row 5 -column 4 -sticky e
    grid $itk_component(sf_manual_l) -row 7 -column 2 -sticky w
    grid $itk_component(sf_manual_r) -row 7 -column 4 -sticky e
    grid $itk_component(sf_del_l) -row 9 -column 2 -sticky w
    grid $itk_component(sf_del_r) -row 9 -column 4 -sticky e
    grid $itk_component(sf_total_l) -row 11 -column 2 -sticky w
    grid $itk_component(sf_total_r) -row 11 -column 4 -sticky e

    grid columnconfigure $itk_component(spotselectionframe) { 0 1 3 5 6 } -minsize 14
    grid columnconfigure $itk_component(spotselectionframe) { 3 } -weight 1
    grid rowconfigure $itk_component(spotselectionframe) { 0 2 4 10 12} -minsize 14
    grid rowconfigure $itk_component(spotselectionframe) { 2 4 10 12 } -minsize 21
    grid rowconfigure $itk_component(spotselectionframe) { 2 }  -weight 1
    grid rowconfigure $itk_component(spotselectionframe) { 12 }  -weight 2
        
    eval itk_initialize $args

}

body Spotfinding::launch { an_image_list } {

    # clear existing image arrays 
    array unset image_objects_by_number *
    array unset image_objects_by_item *
    array unset image_items_by_number *
    array unset image_items_by_object *

    # clear the image tree
    $itk_component(image_tree) item delete all

    # clear the spot canvas
    $itk_component(spotcanvas) delete all

    set l_num_sectors [llength [$::session getSectors]]
    if {$l_num_sectors == 1} {
	buildImageList
    } elseif {$l_num_sectors > 1} {
	buildImageTree
    } else {
	error "Trying to begin indexing without images!"
    }
    
    # find spots on chosen/default images
    # get the first image's number
    set l_first_image_number [lindex [lsort -dictionary [array names image_objects_by_number]] 0]
    # find spots on the first image
    findSpotsIfNecessary $image_objects_by_number($l_first_image_number)
}

body Spotfinding::buildImageList { } {

    # Configure the treectrl with apprpriate column width, and remove buttons/lines
    $itk_component(image_tree) configure -showlines 0 -showbuttons 0
    $itk_component(image_tree) column configure 0 -width 100

    ### Build list of images for tree

    # Choose labelling method depending on number of templates
    if {[llength [$::session getSectors]] > 1} {
	set l_labelMethod "getShortName"
    } else {
	set l_labelMethod "getNumber"
    }

    # loop through session's images...
    foreach i_image [$::session getImages] {
	# create a new item
	set t_item [$itk_component(image_tree) item create]
	# set the item's style
	$itk_component(image_tree) item style set $t_item 0 s1 1 s2 2 s2 3 s2 4 s2 5 s3 6 s4
	# get the label to be used
	set l_label [$i_image $l_labelMethod]
	# get any existing spotlist
	set l_spotlist [$i_image getSpotlist]
	# depending on whether there's a spotlist or not...
	if {$l_spotlist != ""} {
	    # Set state to indicate spots are available from this image (but none checked yet)
	    $itk_component(image_tree) item state set $t_item [list AVAILABLE !CHECKED]
	    # Get the text summary for display in the image tree
	    set l_icon ::img::spotlist
	    set l_auto [format %3d [$l_spotlist getAuto]]
	    set l_manual [format %3d [$l_spotlist getManual]]
	    set l_deleted [format %3d [$l_spotlist getDeleted]]
	    set l_total [format %3d [$l_spotlist getTotal]]
	    # plot the spots in the canvas (tagged by label)
	    $l_spotlist plotSummary $itk_component(spotcanvas) $l_label
	} else {
	    # Set state to indicate spots are NOT available from this image
	    $itk_component(image_tree) item state set $t_item [list !AVAILABLE !CHECKED] 
	    # Make text summary indicating no spots searched for yet
	    set l_icon ::img::plain_image
	    set l_auto " - "
	    set l_manual " - "
	    set l_deleted " - "
	    set l_total " - "
	}
	# update the image icon
	$itk_component(image_tree) item element configure $t_item 0 e_icon -image $l_icon
	# update the text summaries
	$itk_component(image_tree) item text $t_item 0 $l_label 1 $l_auto 2 $l_manual 3 $l_deleted 4 $l_total
	# add the new item to the tree
	$itk_component(image_tree) item lastchild root $t_item
	# Store pointer to image objects and items by bumber, item or object
	set image_objects_by_number([$i_image getNumber]) $i_image
	set image_objects_by_item($t_item) $i_image
	set image_items_by_number([$i_image getNumber]) $t_item
	set image_items_by_object($i_image) $t_item
    }
    # Sort the image tree
    $itk_component(image_tree) item sort root -command [code $this sortSpotFindingResults]
    # Hide all spots as none should be checked yet
    $itk_component(spotcanvas) itemconfigure "all" -image ""
    # Move the spots to avoid the canvas border
    $itk_component(spotcanvas) move "all" 1 1
    # Scale the spot summary canvas
    set l_scale_factor [expr 250.0 / [$i_image getImageHeight]]
    $itk_component(spotcanvas) scale "all" 1 1 $l_scale_factor $l_scale_factor
    
}


body Spotfinding::findSpots { args } {

    # Turn on spotfinding flag
    set spotfinding_flag 1

    # Keep list of images being list
    set images_being_searched [concat $images_being_searched $args]

    # Build array of images being searched by spotfile name
    foreach i_im $args {
	set req_search_results([$i_im getSpotfileName]) $i_im
    }

    # Update the status indicators
#     anigif::anigif [file join $::env(MOSFLM_GUI) images active.gif] $itk_component(activity_l)
#     $itk_component(status_message) configure -text "Finding spots on image [[lindex $args 0] getNumber]"
    
    # Tell mosflm to find spots on the picked images
    $::mosflm findspots $args
}

body Spotfinding::findSpotsIfNecessary { args } {
    # Only actually searches images that don't already have a spotlist,
    #  merely checking the other images for indexing 
    set l_necessary_images {}
    foreach i_image $args {
	if {[$i_image getSpotlist] == ""} {
	    lappend l_necessary_images $i_image
	} else {
	    checkSpotlistInclusion $image_items_by_object($i_image)
	}
    }
    if {$l_necessary_images != {}} {
	findSpots $l_necessary_images
    }
}
    
body Spotfinding::pickFirstImage { } {
    # Clear any currently checkd items
    foreach i_item [$itk_component(image_tree) item children root] {
	uncheckSpotlistInclusion $i_item
    }
    # Get the lowest numbered image
    set l_first_number [lindex [lsort [array names image_objects_by_number]] 0]
    set l_first_image $image_objects_by_number($l_first_number)
    set l_first_item $image_items_by_number($l_first_number)
    # depending on its spotlist availablity...
    if {![$itk_component(image_tree) item state get $l_first_item AVAILABLE]} {
	# search for spots on the image
	findSpots $l_first_image
	# (Image will automatically be checked on search completion)
    } else {
	# Check the item!
	checkSpotlistInclusion $l_first_item
    }
}

body Spotfinding::pickNinetyDegreeImages { } {
}

body Spotfinding::chooseImages { } {
    
    # Set flag to indicate image choosing is going on
    set choosing_images 1
    # Initialize list of images to search
    set chosen_image_numbers {}
    set chosen_search_images {}
    # Get the list of numbers being searched for
    set uncompressed_chosen_images_text [::uncompressNumList $chosen_images_text]
    # Weed out entries that no match exisiting image numbers
    foreach i_num $uncompressed_chosen_images_text {
	# 0 pad number
	set t_num [format %0[string length [lindex [array names image_objects_by_number] 0]]d $i_num]
	if {[lsearch [array names image_objects_by_number] $t_num] > -1} {
	    lappend chosen_image_numbers $t_num
	}
    }
    set chosen_images_text [::compressNumList $chosen_image_numbers]
    $itk_component(image_numbers_e) configure -state disabled
    foreach i_item [$itk_component(image_tree) item children root] {
	uncheckSpotlistInclusion $i_item
    }
    foreach i_num $chosen_image_numbers {
	if {[$itk_component(image_tree) item state get $image_items_by_number($i_num) AVAILABLE]} {
	    checkSpotlistInclusion $image_items_by_number($i_num)
	} else {
	    lappend chosen_search_images $image_objects_by_number($i_num)
	}
    } 
    chooseNextImage
}

body Spotfinding::chooseNextImage { } {
    if {$choosing_images} {
	if {[llength $chosen_search_images] != 0} {
	    findSpots [lindex $chosen_search_images 0]
	    set chosen_search_images [lrange $chosen_search_images 1 end]
	} else {
	    set choosing_images 0
	    $itk_component(image_numbers_e) configure -state normal
	}
    } else {
	set chosen_search_images {}
	$itk_component(image_numbers_e) configure -state normal
    }
}   


body Spotfinding::processSpotfindingResults { a_dom } {
    
    # Check on status of task
    set status_code [$a_dom selectNodes string(/spot_search_response/status/code)]
    if {$status_code == "error"} {
	.m confirm \
	    -type "1button" \
	    -text "Spot search failed, sorry.\n[$a_dom selectNodes string(/spot_search_response/status/message)]" \
	    -button1of1 "Dismiss"
    } else {
	# Get the name of the spot file returned
	set received_file [$a_dom selectNodes string(//spotfile)]
	# Get the pointer to the image it belongs to
	set l_image $req_search_results($received_file)
	# Remove the file from the list we're waiting for
	set l_pos [lsearch $images_being_searched $l_image]
	set images_being_searched [lreplace $images_being_searched $l_pos $l_pos]
	array unset req_search_results $received_file
	# Create a spotlist from the file
	set new_spotlist [namespace current]::[Spotlist \#auto "file" $l_image [$l_image getSpotfileName]]
	#Add an event to the session history recording old and new spotlists
	set old_spotlist [$l_image getSpotlist]
	$::session addHistoryEvent "SpotSearchEvent" "Indexing" [$l_image getFullPathName] $old_spotlist $new_spotlist
	# Update the image's spotlist
	$l_image setSpotlist $new_spotlist
	# Add flag to list of spotlists to be used
	lappend use_spotlists 1
	# If there are no more results awaited
	if {[llength $images_being_searched] == 0} {
	    # Turn status indicators to 'done'
	    set spotfinding_flag 0
# 	    $itk_component(activity_l) configure -image ::img::inactive
# 	    $itk_component(status_message) configure -text ""
	} else {
	    # update status message
	    set l_pos [lsearch $images_list $l_image]
	    if {$l_pos >= "0"} {
		$itk_component(status_message) configure \
		    -text "Finding spots on image [[lindex $images_list [expr $l_pos + 1]] getNumber]"
	    }
	}

	# Update the image tree
	updateSpotFindingResult $l_image

	# Auto-check (for use) the searched image
	checkSpotlistInclusion $image_items_by_object($l_image)

	# Sort the image tree
	$itk_component(image_tree) item sort root -command [code $this sortSpotFindingResults]
	$itk_component(image_tree) yview moveto 0

	# Update summary labels
	updateSummaryLabels

	# Update image viewer with spots, if now available
	.image plotSpots
    }

    # Continue to next image (if image choosing - tested in called method)
    chooseNextImage
}


body Spotfinding::imageTreeClick { w x y } {
    set id [$w identify $x $y]
    if {$id eq ""} {
    } elseif {[lindex $id 0] eq "header"} {
    } else {
	$w activate [$w index [list nearest $x $y]]
	foreach {what item where arg1 arg2 arg3} $id break
	if {[lindex $id 5] == "e_check"} {
	    toggleSpotlistInclusion $item
	}
    }
}

body Spotfinding::imageTreeDoubleClick { w x y } {
    set id [$w identify $x $y]
    if {$id eq ""} {
    } elseif {[lindex $id 0] eq "header"} {
    } else {
	foreach {what item where arg1 arg2 arg3} $id {}
	if {[lindex $id 5] == "e_auto_search"} {
	    set choosing_images "0"
	    findSpots $image_objects_by_item($item)
	} elseif {[lindex $id 5] == "e_manual_search"} {
	    .image editSpots $image_objects_by_item($item)
	} elseif {[lindex $id 5] == "e_check"} {
	    toggleSpotlistInclusion $item
	}
    }
}

body Spotfinding::checkSpotlistInclusion { an_item } {
    # if the spotlist is not available and unchecked don't bother!
    if {(![$itk_component(image_tree) item state get $an_item AVAILABLE]) || [$itk_component(image_tree) item state get $an_item CHECKED]} {
	return
    }
    # get the item's label
    set l_label [$itk_component(image_tree) item text $an_item 0]
    # make the item  cheked...
    $itk_component(image_tree) item state set $an_item CHECKED
    # pick colour according to selectedness
    if {[$itk_component(image_tree) item state get $an_item selected]} {
	set l_colour "blue"
    } else {
	set l_colour "black"
    } 
    # Show spots in canvas 
    $itk_component(spotcanvas) itemconfigure "auto$l_label" -image ::img::summary_spot_${l_colour}_plus
    $itk_component(spotcanvas) itemconfigure "manual$l_label" -image ::img::summary_spot_${l_colour}_cross
    $itk_component(spotcanvas) raise "all$l_label"
    # Update summary labels
    updateSummaryLabels
}

body Spotfinding::uncheckSpotlistInclusion { an_item } {
    # if the spotlist is not available and checked don't bother!
    if {(![$itk_component(image_tree) item state get $an_item AVAILABLE]) || (![$itk_component(image_tree) item state get $an_item CHECKED])} {
	return
    }
    # get the item's label
    set l_label [$itk_component(image_tree) item text $an_item 0]
    # make the item uncheked...
    $itk_component(image_tree) item state set $an_item !CHECKED
    # Hide the spots in the canvas
    $itk_component(spotcanvas) itemconfigure "all$l_label" -image ::img::summary_spot_blank
    # Update summary labels
    updateSummaryLabels
}


body Spotfinding::toggleSpotlistInclusion { an_item } {
    set choosing_images 0
    if {[$itk_component(image_tree) item state get $an_item CHECKED]} {
	uncheckSpotlistInclusion $an_item
    } else {
	checkSpotlistInclusion $an_item
    }
}

body Spotfinding::toggleImageSelection { a_selected { a_deselected "" } } {

    # if the selected item is checked...
    if {($a_selected != "") && [$itk_component(image_tree) item state get $a_selected CHECKED]} {
	# get its label
	set l_label [$itk_component(image_tree) item text $a_selected 0]
	# colour its spots blue
	$itk_component(spotcanvas) itemconfigure "auto$l_label" -image ::img::summary_spot_blue_plus
	$itk_component(spotcanvas) itemconfigure "manual$l_label" -image ::img::summary_spot_blue_cross
	# raise them
	$itk_component(spotcanvas) raise "all$l_label"
    }
    # if the deselected item is checked...
    if {($a_deselected != "") && [$itk_component(image_tree) item state get $a_deselected CHECKED]} {
	# get its label
	set l_label [$itk_component(image_tree) item text $a_deselected 0]
	# colour its spots black
	$itk_component(spotcanvas) itemconfigure "auto$l_label" -image ::img::summary_spot_black_plus
	$itk_component(spotcanvas) itemconfigure "manual$l_label" -image ::img::summary_spot_black_cross    
    }
}

body Spotfinding::updateSpotlists { an_image } {
    # Don't bother if we can't find the image item
    if {[array get image_items_by_object $an_image] != ""} {
	# Get the label
	set l_label [$an_image getNumber]
	# get any existing spotlist
	set l_spotlist [$an_image getSpotlist]
	# depending on whether there's a spotlist or not...
	if {$l_spotlist != ""} {
	    # Manual spot editing implies we want to use these spots!
	    $itk_component(image_tree) item state set $image_items_by_object($an_image) [list AVAILABLE CHECKED]
	    # Get the text summary for display in the image tree
	    set l_icon ::img::spotlist
	    set l_auto [format %3d [$l_spotlist getAuto]]
	    set l_manual [format %3d [$l_spotlist getManual]]
	    set l_deleted [format %3d [$l_spotlist getDeleted]]
	    set l_total [format %3d [$l_spotlist getTotal]]
	    # plot the spots in the canvas (tagged by label)
	    $l_spotlist plotSummary $itk_component(spotcanvas) $l_label
	    # Move the spots to avoid the canvas border
	    $itk_component(spotcanvas) move "all$l_label" 1 1
	    # Scale the spot summary canvas
	    set l_scale_factor [expr 250.0 / [$an_image getImageHeight]]
	    $itk_component(spotcanvas) scale "all$l_label" 1 1 $l_scale_factor $l_scale_factor
	    # If the item is checked for inclusion...
	    if {[$itk_component(image_tree) item state get $image_items_by_object($an_image) CHECKED]} {
		# If the item is currently selected...
		if {[$itk_component(image_tree) item state get $image_items_by_object($an_image) selected]} {
		    # colour its spots blue
		    $itk_component(spotcanvas) itemconfigure "auto$l_label" -image ::img::summary_spot_blue_plus
		    $itk_component(spotcanvas) itemconfigure "manual$l_label" -image ::img::summary_spot_blue_cross    
		}
	    } else {
		$itk_component(spotcanvas) itemconfigure "manual$l_label" -image ""
	    }
	} else {
	    # Set state to indicate spots are NOT available from this image
	    $itk_component(image_tree) item state set $image_items_by_object($an_image) [list !AVAILABLE !CHECKED] 
	    # Make text summary indicating no spots searched for yet
	    set l_icon ::img::plain_image
	    set l_auto " - "
	    set l_manual " - "
	    set l_deleted " - "
	    set l_total " - "
	}
	# update the image icon
	$itk_component(image_tree) item element configure $image_items_by_object($an_image) 0 e_icon -image $l_icon
	# update the text summaries
	$itk_component(image_tree) item text $image_items_by_object($an_image) 1 $l_auto 2 $l_manual 3 $l_deleted 4 $l_total
    }
    # Update summary canvas
    updateSpotlistSelection
    # Update summary labels
    updateSummaryLabels
}

body Spotfinding::updateSpotFindingResult { a_image } {
    # Don't bother if the image isn't in the wizard's list!
    if {![info exists image_items_by_object($a_image)]} {
	return
    }
    # get the image's item
    set l_item $image_items_by_object($a_image)
    # get the image's spotlist
    set l_spotlist [$a_image getSpotlist]
    # get the image's label in the image tree
    set l_label [$itk_component(image_tree) item text $l_item 0]
    # Delete any plotted spots for the image
    $itk_component(spotcanvas) delete "all$l_label"
    if {$l_spotlist != ""} {
	$itk_component(image_tree) item state set $l_item AVAILABLE
	set l_icon ::img::spotlist
	set l_auto [format %3d [$l_spotlist getAuto]]
	set l_manual [format %3d [$l_spotlist getManual]]
	set l_deleted [format %3d [$l_spotlist getDeleted]]
	set l_total [format %3d [$l_spotlist getTotal]]
	# plot summary
	$l_spotlist plotSummary $itk_component(spotcanvas) $l_label
	# Move the spots to avoid the canvas border
	$itk_component(spotcanvas) move "all$l_label" 1 1
	# Scale the spot summary canvas
	set l_scale_factor [expr 250.0 / [$a_image getImageHeight]]
	$itk_component(spotcanvas) scale "all$l_label" 1 1 $l_scale_factor $l_scale_factor
	# If the item is currently selected...
	if {[$itk_component(image_tree) item state get $l_item selected]} {
	    # colour its spots blue
	    $itk_component(spotcanvas) itemconfigure "auto$l_label" -image ::img::summary_spot_blue_plus
	    $itk_component(spotcanvas) itemconfigure "manual$l_label" -image ::img::summary_spot_blue_cross    
	}
    } else {
	$itk_component(image_tree) item state set $l_item !AVAILABLE
	set l_icon ::img::plain_image
	set l_auto " - "
	set l_manual " - "
	set l_deleted " - "
	set l_total " - "
    }
    $itk_component(image_tree) item element configure $l_item 0 e_icon -image $l_icon
    $itk_component(image_tree) item text $l_item 1 $l_auto 2 $l_manual 3 $l_deleted 4 $l_total
}

body Spotfinding::sortSpotFindingResults { a_item b_item } {
    set a_available [$itk_component(image_tree) item state get $a_item AVAILABLE]
    set b_available [$itk_component(image_tree) item state get $b_item AVAILABLE]
    if {$a_available && !$b_available} {
	return -1
    } elseif {!$a_available && $b_available} {
	return +1
    } else {
	set a_image_num [$image_objects_by_item($a_item) getNumber]
	set b_image_num [$image_objects_by_item($b_item) getNumber]
	if {$a_image_num < $b_image_num} {
	    return -1
	} elseif {$a_image_num > $b_image_num} {
	    return +1
	} else {
	    return 0
	}
    }
}

body Spotfinding::editSpots { } {
    .image openImage [lindex $images_list [$itk_component(imageslist) curselection]]
}

body Spotfinding::createImageCheck { tbl row col w } {
    gEmbeddedCheck $w [lindex $use_spotlists $row] \
	-command [code $this updateSpotlistInclusions $row]
}

body Spotfinding::updateSpotlistInclusions { a_row a_value } {
    # update the list of which spotlists to use
    set use_spotlists [lreplace $use_spotlists $a_row $a_row $a_value]
    # Update summary canvas
    updateSpotlistSelection
    # Update summary labels
    updateSummaryLabels
}

body Spotfinding::updateSpotlistSelection {  } {
    # initialize the index used to count through spot lists
    set i_index 0
    # loop through images 
    foreach i_image $images_list {
	# if the images's spotlist is to be used...
	if {[lindex $use_spotlists $i_index] == "1"} {
	    if {[$itk_component(imageslist) curselection] == $i_index} {
		# show the markers
		$itk_component(spotcanvas) itemconfigure "auto[$i_image getNumber]" -image ::img::summary_spot_blue_plus
		$itk_component(spotcanvas) itemconfigure "manual[$i_image getNumber]" -image ::img::summary_spot_blue_cross
		$itk_component(spotcanvas) raise "all[$i_image getNumber]"
	    } else {
		$itk_component(spotcanvas) itemconfigure "auto[$i_image getNumber]" -image ::img::summary_spot_black_plus
		$itk_component(spotcanvas) itemconfigure "manual[$i_image getNumber]" -image ::img::summary_spot_black_cross
	    }
	} else {
	    $itk_component(spotcanvas) itemconfigure "all[$i_image getNumber]" -image ::img::summary_spot_blank
	}
	incr i_index
    }
} 

body Spotfinding::updateSummaryLabels { } {
    set l_checked_image_numbers {}
    set l_images 0
    set l_auto 0
    set l_manual 0
    set l_del 0
    set l_total 0
    set i_index 0
    foreach i_item [$itk_component(image_tree) item children root] {
	if {[$itk_component(image_tree) item state get $i_item CHECKED]} {
	    set l_image $image_objects_by_item($i_item)
	    lappend l_checked_image_numbers [$l_image getNumber]
	    set l_spotlist [$l_image getSpotlist]
	    incr l_images
	    incr l_auto [$l_spotlist getAuto]
	    incr l_manual [$l_spotlist getManual]
	    incr l_del [$l_spotlist getDeleted]
	    incr l_total [$l_spotlist getTotal]
	}
    }
    set chosen_images_text [compressNumList $l_checked_image_numbers]
    $itk_component(sf_images_r) configure -text $l_images
    $itk_component(sf_auto_r) configure -text $l_auto
    $itk_component(sf_manual_r) configure -text $l_manual
    $itk_component(sf_del_r) configure -text $l_del
    $itk_component(sf_total_r) configure -text $l_total
#     if {$l_total == "0"} {
# 	$itk_component(next) configure -state disabled
#     } else {
# 	$itk_component(next) configure -state normal
#     }
}

usual Spotfinding { } 
